@@ -13,11 +13,12 @@
 namespace fcitx {
 
 class InputContext;
-class CandidateList;
 class PageableCandidateList;
 class BulkCandidateList;
 class ModifiableCandidateList;
 class CursorMovableCandidateList;
+class CursorModifiableCandidateList;
+class BulkCursorCandidateList;
 
 class CandidateListPrivate;
 
@@ -47,12 +48,28 @@ public:
     bool isPlaceHolder() const;
     bool hasCustomLabel() const;
     const Text &customLabel() const;
+    /**
+     * Return comment corresponding to the candidate.
+     *
+     * @return value of comment.
+     * @since 5.1.9
+     */
+    const Text &comment() const;
+    /**
+     * Return text with comment.
+     *
+     * @param separator separator between text and comment.
+     * @return value of comment.
+     * @since 5.1.9
+     */
+    Text textWithComment(std::string separator = " ") const;
 
 protected:
     void setText(Text text);
     void setPlaceHolder(bool placeHolder);
     void resetCustomLabel();
     void setCustomLabel(Text text);
+    void setComment(Text comment);
 
 private:
     std::unique_ptr<CandidateWordPrivate> d_ptr;
@@ -77,12 +94,16 @@ public:
     BulkCandidateList *toBulk() const;
     ModifiableCandidateList *toModifiable() const;
     CursorMovableCandidateList *toCursorMovable() const;
+    CursorModifiableCandidateList *toCursorModifiable() const;
+    BulkCursorCandidateList *toBulkCursor() const;
 
 protected:
     void setPageable(PageableCandidateList *list);
     void setBulk(BulkCandidateList *list);
     void setModifiable(ModifiableCandidateList *list);
     void setCursorMovable(CursorMovableCandidateList *list);
+    void setCursorModifiable(CursorModifiableCandidateList *list);
+    void setBulkCursor(BulkCursorCandidateList *list);
 
 private:
     std::unique_ptr<CandidateListPrivate> d_ptr;
@@ -112,6 +133,11 @@ public:
     virtual void nextCandidate() = 0;
 };
 
+class FCITXCORE_EXPORT CursorModifiableCandidateList {
+public:
+    virtual void setCursorIndex(int index) = 0;
+};
+
 // useful for virtual keyboard
 class FCITXCORE_EXPORT BulkCandidateList {
 public:
@@ -155,6 +181,12 @@ public:
     void select(InputContext *) const override {}
 };
 
+class FCITXCORE_EXPORT BulkCursorCandidateList {
+public:
+    virtual int globalCursorIndex() const = 0;
+    virtual void setGlobalCursorIndex(int index) = 0;
+};
+
 class DisplayOnlyCandidateListPrivate;
 
 class FCITXCORE_EXPORT DisplayOnlyCandidateList : public CandidateList {
@@ -229,6 +261,14 @@ public:
      */
     int globalCursorIndex() const;
 
+    /**
+     * Set cursor index on current page.
+     *
+     * @param index index on current page;
+     * @since 5.1.9
+     */
+    void setCursorIndex(int index);
+
     // CandidateList
     const fcitx::Text &label(int idx) const override;
     const CandidateWord &candidate(int idx) const override;
